
import org.drools.jsr94.rules.RuleServiceProviderImpl;
import org.drools.decisiontable.DecisionTableLoader;

import javax.rules.RuleServiceProviderManager;
import javax.rules.RuleServiceProvider;
import javax.rules.StatelessRuleSession;
import javax.rules.RuleRuntime;
import javax.rules.admin.RuleAdministrator;
import javax.rules.admin.LocalRuleExecutionSetProvider;
import javax.rules.admin.RuleExecutionSet;
import java.io.InputStream;
import java.util.ArrayList;

public class Exchange
{
    private final String  RULE_SERVICE_PROVIDER = "http://drools.org/";
    private final String RULE_URI = "CollectorRules.drl";
    private StatelessRuleSession _statelessRuleSession;
    private RuleAdministrator _ruleAdministrator;

    private Collector _seller;
    private Collector _buyer;

    public boolean okToExchange = true;
    private boolean clean = false;
    private int _quantity;

    public Exchange() throws Exception
    {
        prepare();
    }

    protected void finalize() throws Throwable
    {
        if (!clean)  { cleanUp(); }
    }

    public Collector getSeller() { return _seller; }
    public Collector getBuyer() { return _buyer; }
    public int getQuantity() { return _quantity; }
    public void refuseExchange() { okToExchange = false; }
    public void oKExchange() { okToExchange = true; }

    private void prepare() throws Exception
    {
        RuleServiceProviderManager.registerRuleServiceProvider(
                RULE_SERVICE_PROVIDER, RuleServiceProviderImpl.class );

        RuleServiceProvider ruleServiceProvider =
                RuleServiceProviderManager.getRuleServiceProvider(RULE_SERVICE_PROVIDER);

        _ruleAdministrator = ruleServiceProvider.getRuleAdministrator( );

        LocalRuleExecutionSetProvider ruleSetProvider =
                _ruleAdministrator.getLocalRuleExecutionSetProvider(null);

        InputStream rules = Exchange.class.getResourceAsStream(RULE_URI);
        RuleExecutionSet ruleExecutionSet =
                ruleSetProvider.createRuleExecutionSet(rules, null);

        _ruleAdministrator.registerRuleExecutionSet(RULE_URI, ruleExecutionSet, null);

        RuleRuntime ruleRuntime = ruleServiceProvider.getRuleRuntime();

        _statelessRuleSession =
                (StatelessRuleSession) ruleRuntime.createRuleSession(
                        RULE_URI, null, RuleRuntime.STATELESS_SESSION_TYPE );
    }

    public void cleanUp() throws Exception
    {
        clean = true;
        _statelessRuleSession.release( );
        _ruleAdministrator.deregisterRuleExecutionSet(RULE_URI, null);
    }

    public boolean tradeFromSellerToBuyer(Collector seller, Collector buyer,
                                          int quantity) throws Exception
    {
        _seller = seller;
        _buyer = buyer;
        _quantity = quantity;

        okToExchange = true;

        //We will apply rules and then if it is OK, we will trade.
        ArrayList objectList = new ArrayList();
        objectList.add(seller);
        objectList.add(buyer);
        objectList.add(this);

        _statelessRuleSession.executeRules(objectList);

        if (okToExchange)
        {
            seller.sell(quantity);
            buyer.buy(quantity);
        }

        return okToExchange;
    }
}
